/*
    opinionDynamics 0.1: ObserverSwarm.m
    Copyright (C) 2005 by Gary McKeown
	
	This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation.

    Contact information:
		Gary McKeown,
		School of Psychology,
		Queens University of Belfast,
		Belfast BT7 1NN,
		Northern Ireland, 
		UK.
	g.mckeown@qub.ac.uk
*/


#import "ObserverSwarm.h"
#import "ModelSwarm.h"
#import "Parameters.h"
#import <collections.h>
#import <objectbase.h>

@implementation ObserverSwarm

+createBegin: (id) aZone {
	ObserverSwarm * obj;
	id <ProbeMap> probeMap;

	obj = [super createBegin: aZone];
	obj->displayFrequency = 1;

	probeMap = [EmptyProbeMap createBegin: aZone];
	[probeMap setProbedClass: [self class]];
	probeMap = [probeMap createEnd];

	[probeMap addProbe: [probeLibrary getProbeForVariable: "displayFrequency"
					inClass: [self class]]];
					
	[probeMap addProbe: [probeLibrary getProbeForMessage: "takeScreenShot"
					inClass: [self class]]];

	[probeLibrary setProbeMap: probeMap For: [self class]];

	return obj;
}

-createEnd {
	return [super createEnd];
}

// Create the objects used in the display of the model. 
-buildObjects {
	id modelZone;					  // zone for model.

	[super buildObjects];

	modelZone = [Zone create: self];
	modelSwarm = [ModelSwarm create: modelZone];
  
	CREATE_ARCHIVED_PROBE_DISPLAY (self);

    [controlPanel setStateStopped];

	// Check now if the user hit the quit button: if so, abort.
	if ([controlPanel getState] == ControlStateQuit) return self;

	[modelSwarm buildObjects];

	colormap = [Colormap create: self];
  
  	int i;
	for (i = 1; i <= 200; i++) {
		[colormap setColor: i ToRed: (double)i / 200 Green: 0 Blue: 0];
	}

	// Create the window for displaying agents
	worldRaster = [ZoomRaster createBegin: self];
	SET_WINDOW_GEOMETRY_RECORD_NAME (worldRaster);
	worldRaster = [worldRaster createEnd];
	[worldRaster setColormap: colormap];
	[worldRaster setZoomFactor: 6];
	[worldRaster setWidth: [(Parameters *)arguments worldSize] Height: [(Parameters *)arguments worldSize]];
	[worldRaster setWindowTitle: "Opinion World"];
	[worldRaster pack];

	worldDisplay = [Object2dDisplay createBegin: self ];
	[worldDisplay setDisplayWidget: worldRaster];
	[worldDisplay setDiscrete2dToDisplay: [[modelSwarm getWorld] getObjectGrid]];
	[worldDisplay setObjectCollection: [modelSwarm getAgentList]];
	[worldDisplay setDisplayMessage: M(drawSelfOn:)];   
	worldDisplay = [worldDisplay createEnd];

	// This allows the user to right-click on the display to probe agents
	[worldRaster setButton: ButtonRight Client: worldDisplay Message: M(makeProbeAtX:Y:)];

//	Graph to display the mean opinion
	meanGraph = [EZGraph createBegin: self];
	SET_WINDOW_GEOMETRY_RECORD_NAME (meanGraph);
	[meanGraph setTitle: "Current mean opinion"];
	[meanGraph setAxisLabelsX: "time" Y: "number"];
	meanGraph = [meanGraph createEnd];

	[meanGraph createAverageSequence: "opinion"
	withFeedFrom: [modelSwarm getAgentList]
	andSelector: M(getOpinion)] ;
  
//	Graph to display the opinion standard deviation
	stdDevGraph = [EZGraph createBegin: self];
	SET_WINDOW_GEOMETRY_RECORD_NAME (stdDevGraph);
	[stdDevGraph setTitle: "Current opinion standard deviation"];
	[stdDevGraph setAxisLabelsX: "time" Y: "number"];
	stdDevGraph = [stdDevGraph createEnd];

	[stdDevGraph createStdDevSequence: "Std Dev"
	withFeedFrom: [modelSwarm getAgentList]
	andSelector: M(getOpinion)];
	
//	Graph to display the Weisbuch et al indicator
	indicatorGraph = [EZGraph createBegin: self];
	SET_WINDOW_GEOMETRY_RECORD_NAME (indicatorGraph);
	[indicatorGraph setTitle: "indicator y"];
	[indicatorGraph setAxisLabelsX: "time" Y: "number"];
	indicatorGraph = [indicatorGraph createEnd];

	[indicatorGraph createSequence: "indicator"
	withFeedFrom: modelSwarm
	andSelector: M(getIndicator)];

	return self;
}  

-buildActions {
	[super buildActions];

	// Tell the model Swarm to build it's schedules
	[modelSwarm buildActions];

	// Then create a group of actions for the observer
	displayActions = [ActionGroup create: self ];

	[displayActions createActionTo: probeDisplayManager message: M(update)];
	[displayActions createActionTo: actionCache         message: M(doTkEvents)];
	[displayActions createActionTo: self                message: M(eraseRaster)];
	[displayActions createActionTo: worldDisplay        message: M(display)];
	[displayActions createActionTo: worldRaster         message: M(drawSelf)];
	[displayActions createActionTo: meanGraph	message:M(step)];
	[displayActions createActionTo: stdDevGraph	message:M(step)];
	[displayActions createActionTo: indicatorGraph	message:M(step)];
	// Put these actions on a schedule to be repeated at a certain frequency
	displaySchedule = [Schedule createBegin: self];
	[displaySchedule setRepeatInterval: displayFrequency]; // note frequency!
	displaySchedule = [displaySchedule createEnd];
	[displaySchedule at: 0 createAction: displayActions];

	//This is a one-time schedule that pops up and 
	//stops the simulation when it reaches the "duration".
    stopSchedule = [Schedule createBegin: [self getZone]];
    [stopSchedule setRepeatInterval: 1];
    stopSchedule = [stopSchedule createEnd];
    [stopSchedule at: 0 createActionTo: self message: M(stopRunning)];

	return self;
}  

-activateIn: (id) swarmContext {

	[super activateIn: swarmContext];
	[displaySchedule activateIn: self];
	[modelSwarm activateIn: self];
	[stopSchedule activateIn: self];
	
	return [self getSwarmActivity];
}

- stopRunning
{
	currentTime = getCurrentTime();

	if(currentTime == getInt((Parameters*)arguments,"experimentDuration"))
	{
		#ifdef UNATTENDED
//		[self takeScreenShot];
		// This makes the simulation activity stop and Swarm quits.	
		[controlPanel setStateQuit];
		#else

		// If you don't define UNATTENDED, then the simulation just pauses here,
		// and user intervention with the control panel can start it again.
		[controlPanel setStateStopped];
		[modelSwarm stopRunning];

		#endif
	}

  return self;
}

- takeScreenShot
{
	char filename[40];
	id aPix;
	sprintf (filename, "screenRun%03dTime%07ld.png",getInt((Parameters*)arguments,"run"), currentTime);
	[actionCache doTkEvents];

	aPix =[ Pixmap createBegin: self];
	[aPix setWidget: nil];
	[aPix setDecorationsFlag: NO];
	aPix = [aPix createEnd];
	[aPix save: filename];
	[aPix drop];

	return self;
}

-eraseRaster {
	// A method that overrides the default
	// black background for raster and paints
	// the raster white instead
	[worldRaster fillRectangleX0: 0
							Y0: 0
							X1: [(Parameters *)arguments worldSize]
							Y1: [(Parameters *)arguments worldSize]
							Color: 21]; 
	return self;
}

- (void)drop
{
	[modelSwarm drop];
}


@end
